﻿using System;
using System.IO;
using System.Xml.Serialization;
using Tobbretegu.Model;
using Windows.UI.Xaml.Controls;

namespace Tobbretegu.View
{
    public sealed partial class NotesListView : UserControl
    {
        private static readonly object _syncLock = new object();

        private NoteList allNotes;
        private UserList allUsers;

        /// <summary>
        /// The user currently selected as tab.
        /// </summary>
        private User SelectedUser = null;

        /// <summary>
        /// The notes for the selected user; filtered from all notes.
        /// </summary>
        private NoteList DisplayedList { get; set; } = new NoteList();

        public NotesListView()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// Loads data from the files.
        /// </summary>
        private void NavView_Loaded(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            // read files
            this.allNotes = loadNotesFromFile();
            this.allUsers = loadUsersFromFile();

            // add tab pages
            foreach (var user in allUsers)
                navView.MenuItems.Insert(navView.MenuItems.Count - 1, new NavigationViewItem() { Tag = user.Id, Content = user.Name });

            // select the first user by default
            navView.SelectedItem = navView.MenuItems[0];
        }

        /// <summary>
        /// When a user name tab is selected.
        /// </summary>
        private void NavView_SelectionChanged(NavigationView sender, NavigationViewSelectionChangedEventArgs args)
        {
            // selected user's id
            var userId = args.SelectedItemContainer.Tag as Guid?;
            if (!userId.HasValue)
                return;

            // refresh the selected user name on the UI
            foreach (var u in allUsers)
            {
                if (u.Id == userId)
                    SelectedUser = u;
            }
            tbUserName.Text = SelectedUser.Name;

            // refresh the displayed list of notes on the UI
            DisplayedList.Clear();
            foreach (var item in filterNotes(SelectedUser.Id))
                DisplayedList.Add(item);
        }

        public NoteList filterNotes(Guid userId)
        {
            var list = new NoteList();
            foreach (var item in allNotes)
            {
                if (item.Assignee == userId)
                    list.Add(item);
            }
            return list;
        }

        /// <summary>
        /// Delet button of a note
        /// </summary>
        private void BtnDelete_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            // this is the note to delete
            var note = (Note)((Button)sender).DataContext;

            // remove from the notes
            allNotes.Remove(note);

            // remove from the currently displayed list
            DisplayedList.Remove(note);
        }

        /// <summary>
        /// Add new note.
        /// </summary>
        private void BtnAddNewNote_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            // create a new note with dummy date
            var newNote = new Note()
            {
                ID = Guid.NewGuid(),
                Assignee = SelectedUser.Id,
                Title = "Add the title here",
                Description = "Add to text here"
            };

            // add to notes list
            allNotes.Add(newNote);
            saveNotesToFile();

            // add to currently displayed list
            DisplayedList.Add(newNote);
        }

        /// <summary>
        /// Save existing note
        /// </summary>
        private void BtnSave_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            saveNotesToFile();
        }

        /// <summary>
        /// Change current user name.
        /// </summary>
        private void BtnChangeName_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            // make sure user has a name
            if (tbUserName.Text.Length == 0)
                return;

            // update the users list
            string newUserName = tbUserName.Text;
            foreach (var u in allUsers)
            {
                if (u.Id == SelectedUser.Id)
                    u.Name = newUserName;
            }
            saveUsersToFile();

            // update the displayed name on the UI
            SelectedUser.Name = newUserName;
            ((NavigationViewItem)navView.SelectedItem).Content = newUserName;
        }

        /// <summary>
        /// Add a new user.
        /// </summary>
        private void BtnAddNewUser_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            // create a new user
            string newUserName = getNewUserName();
            var newUser = new User()
            {
                Id = Guid.NewGuid(),
                Name = newUserName,
            };

            // add to users list
            allUsers.Add(newUser);
            saveUsersToFile();

            // add menu item on UI
            navView.MenuItems.Insert(navView.MenuItems.Count - 1, new NavigationViewItem() { Tag = newUser.Id, Content = newUser.Name });
            navView.SelectedItem = navView.MenuItems[navView.MenuItems.Count - 2];
        }

        /// <summary>
        /// Generates random new unique name for a user.
        /// </summary>
        private string getNewUserName()
        {
            string newUserName;
            int counter = 1;
            while (true)
            {
                newUserName = $"User {counter++}";
                if (!usernameExists(newUserName))
                    break;
            }

            return newUserName;
        }

        /// <summary>
        /// checks if a user with the same name already exists
        /// </summary>
        private bool usernameExists(string newUserName)
        {
            foreach (var u in allUsers)
            {
                if (u.Name == newUserName)
                    return true;
            }
            return false;
        }

        private UserList loadUsersFromFile()
        {
            string path = Path.Combine(Windows.Storage.ApplicationData.Current.LocalFolder.Path, "users.xml");
            lock (_syncLock)
            {
                // read the file content
                if (File.Exists(path))
                {
                    try
                    {
                        var serializer = new XmlSerializer(typeof(UserList));
                        using (var fs = File.OpenRead(path))
                            return (UserList)serializer.Deserialize(fs);
                    }
                    catch
                    {
                        // if the file is invalid, fall back to defaults
                    }
                }

                // provides defaults for debugging
                return new UserList()
                {
                    new User()
                    {
                        Id = Guid.Parse("87923C8F-17FC-4EDF-99D3-68568BFDB93B"),
                        Name = "John",
                    },
                    new User()
                    {
                        Id = Guid.Parse("8CCB7032-7099-4D70-AE50-26EEC9F8EE8E"),
                        Name = "Mary",
                    }
                };
            }
        }

        private void saveUsersToFile()
        {
            string path = Path.Combine(Windows.Storage.ApplicationData.Current.LocalFolder.Path, "users.xml");
            lock (_syncLock)
            {
                var serializer = new XmlSerializer(typeof(UserList));
                using (var fs = File.Create(path))
                    serializer.Serialize(fs, allUsers);
            }
        }

        private NoteList loadNotesFromFile()
        {
            string path = Path.Combine(Windows.Storage.ApplicationData.Current.LocalFolder.Path, "notes.xml");
            lock (_syncLock)
            {
                if (File.Exists(path))
                {
                    // read the file content
                    try
                    {
                        var serializer = new XmlSerializer(typeof(NoteList));
                        using (var fs = File.OpenRead(path))
                            return (NoteList)serializer.Deserialize(fs);
                    }
                    catch
                    {
                        // if the file is invalid, fall back to defaults
                    }
                }


                // provide default data for debugging
                return new NoteList()
                {
                    new Note()
                    {
                        ID = Guid.NewGuid(),
                        Title = "Otthoni teendők",
                        Description = @"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Suspendisse efficitur enim a mauris gravida elementum. Fusce interdum fermentum nisi, sed tempor magna convallis eu. Suspendisse imperdiet accumsan pharetra. Cras vel nunc sit amet dui volutpat sodales. Sed in felis facilisis, interdum purus in, vestibulum neque. Suspendisse at magna cursus, pulvinar sem non, viverra mauris. Nulla egestas maximus sem, ultricies eleifend lorem pulvinar sit amet. Praesent tincidunt dapibus ante et mattis. Morbi vestibulum sollicitudin venenatis. Mauris finibus, dolor nec aliquet commodo, est ipsum pellentesque dui, et pretium felis est eu sapien.",
                        Assignee = Guid.Parse("87923C8F-17FC-4EDF-99D3-68568BFDB93B")
                    },
                    new Note()
                    {
                        ID = Guid.NewGuid(),
                        Title = "Házi feladatok",
                        Description = @"Phasellus ullamcorper nibh lorem, sit amet accumsan libero efficitur blandit. In non eros viverra, porttitor nunc a, finibus ante. Phasellus mattis leo in luctus accumsan. Nam in leo et nisi sodales posuere sed a purus.",
                        Assignee = Guid.Parse("8CCB7032-7099-4D70-AE50-26EEC9F8EE8E")
                    }
                };
            }
        }

        private void saveNotesToFile()
        {
            string path = Path.Combine(Windows.Storage.ApplicationData.Current.LocalFolder.Path, "notes.xml");
            lock (_syncLock)
            {
                var serializer = new XmlSerializer(typeof(NoteList));
                using (var fs = File.Create(path))
                    serializer.Serialize(fs, allNotes);
            }
        }
    }
}
